'use strict';

Object.defineProperty(exports, '__esModule', { value: true });

var codemirror6 = require('codemirror-6');

var changeToOps = function (path, transaction) { return function (change) {
    var ops = [];
    var p = path.concat([change.from]);
    // String delete
    if (change.from !== change.to) {
        ops.push({
            p: p,
            sd: transaction.startState.doc.slice(change.from, change.to)
        });
    }
    // String insert
    var joined = change.text.join('\n');
    if (joined.length) {
        ops.push({
            p: p,
            si: joined
        });
    }
    return ops;
}; };
var transactionToOps = function (path, transaction) {
    return transaction.changes.changes
        .map(changeToOps(path, transaction))
        .reduce(function (accumulator, ops) { return accumulator.concat(ops); }, []);
};

var siToText = function (si) { return si.split('\n'); };
var opToChange = function (transaction, op) {
    var from = op.p[op.p.length - 1];
    // String insert
    if (op.si !== undefined) {
        return transaction.change(new codemirror6.Change(from, from, siToText(op.si)));
    }
    // String delete
    if (op.sd !== undefined) {
        return transaction.change(new codemirror6.Change(from, from + op.sd.length, ['']));
    }
    throw new Error('Invalid string op.');
};
var opsToTransaction = function (path, state, ops) {
    return ops.reduce(opToChange, state.transaction);
};

var OTState = /** @class */ (function () {
    function OTState() {
    }
    return OTState;
}());
var otPlugin = function (path, emitOps) { return new codemirror6.Plugin({
    state: new codemirror6.StateField({
        init: function (editorState) {
            return new OTState();
        },
        apply: function (transaction, state, editorState) {
            var ops = transactionToOps(path, transaction);
            if (ops.length > 0) {
                emitOps(ops);
            }
            return state;
        },
        debugName: 'otPluginState'
    })
}); };

exports.transactionToOps = transactionToOps;
exports.opsToTransaction = opsToTransaction;
exports.otPlugin = otPlugin;
//# sourceMappingURL=codemirror-ot.js.map
